"use strict";

// 弾塑性応答解析
class response {
    constructor() {
        // 解析情報を初期化
        this.plastic = 0; // 弾塑性特性 = 0: 弾性, 1:Bi-Linear, 2:Tri-Linear
        this.beta = 0.25; // Newmarkのβの値
        // 地震波情報を初期化
        this.wave = {}
        this.wave.name = "ELCENTRO"; // 地震波の名前
        this.wave.acc = -1; // 地震波の最大加速度. -1:原データの値
        this.wave.start = -1; // 地震波の解析開始時刻. -1:原データの値
        this.wave.end = -1; // 地震波の解析終了時刻. -1:原データの値
        this.wave.pitch = -1; // 地震波の時間の刻み幅. -1:原データの値
        // 減衰情報を初期化
        this.damp = {}
        this.damp.type = 0; // 減衰定数 0:剛性に比例, 1:質量に比例, 2:レーリー減衰 
        this.damp.damp1 = 0.03; // 1次減衰定数
        this.damp.damp2 = 0; // 2次減衰定数

        this.nodeData = []; // 各質点の入力データ
        this.maxNode = 0; // 質点数
        this.valMode = []; // 各モードの情報
        this.maxMode = 5; // モードの最大計算次数
        this.error = ""; // エラー情報
        this.G_ACC = 9.80665; // 重力加速度
        // 最大値格納用配列
        this.maxAcc = []; // 絶対加速度
        this.maxVel = []; // 速度
        this.maxDisp = []; // 変位
        this.maxQi = []; // 層せん断力
        this.maxSi = []; // 層間変位
        this.maxTi = []; // 転倒モーメント
        this.maxCi = []; // 層せん断力係数
        this.maxDi = []; // 層間変形角
        this.maxYi = []; // 塑性率
    }
    setCondition({plastic = 0, beta = 0.25, dampType = 0, damp1 = 0.03, damp2 = 0}) {
        // 解析情報のセット
        this.plastic = plastic;
        this.beta = beta;
        this.damp.type = dampType; 
        this.damp.damp1 = damp1;
        this.damp.damp2 = damp2;
    }
    setWave({name = "ELCENTRO", acc = -1, start = -1, end = -1, pitch = -1}) {
        // 地震波情報のセット
        this.wave.name = name; 
        this.wave.acc = acc; 
        this.wave.start = start; 
        this.wave.end = end; 
        this.wave.pitch = pitch; 
    }
    addNodeData({id = 0, name = "", weight = 0, height = 0, force1 = 0, delta1 = 0, force2 = 0, delta2 = 0, beta = 0.01}) {
        // 各質点のデータをnodeData[]に格納
        if (id > 0) {
            let data = {};
            data.id = id; // 質点番号
            data.name = name; // 階名称
            data.weight = weight; // 重量(kN)
            data.height = height; // 階高(mm)
            data.force1 = force1; // 第1折れ点の力(kN)
            data.delta1 = delta1; // 第1折れ点の変位(mm)
            data.force2 = force2; // 第2折れ点の力(kN)
            data.delta2 = delta2; // 第2折れ点の変位(mm)
            data.beta = (beta > 0.001) ? beta : 0.001; // 塑性化後の剛性低下
            for (let i = 0; i < this.nodeData.length; i++) {
                if (this.nodeData[i].id == id) {
                    // id が指定済み
                    this.nodeData[i] = data;
                    return;
                }
            }
            this.nodeData.push(data);
            if (id > this.maxNode) this.maxNode = id; 
        }
    }
    calcMain() {
        // 計算
        // 戻り値 = 0:異常終了, 1:正常終了
        let mass = []; // 質量
        let height = []; // 高さ
        let stiff = []; // 剛性
        // [][0]:初期剛性, [][1]:第2剛性, [][2]第1折点の変位:, [][3]:第3剛性, [][4]:第2折点の変位
        const [K1, K2, D1, K3, D2] = [0, 1, 2, 3, 4];
        let node = this.maxNode;
        initArray1(mass, node);
        initArray1(height, node);
        initArray2(stiff, node, 5);
        let i;
        const [BI_LINEAR, TRI_LINEAR] = [1, 2];
        for (i = 0; i < this.nodeData.length; i++) {
            let index = this.nodeData[i].id - 1;
            mass[index] = this.nodeData[i].weight / (100.0 * this.G_ACC) / this.G_ACC; // 重量 -> 質量 t*s^2/cm
            height[index] = 0.1 * this.nodeData[i].height; // mm -> cm
            if ((this.nodeData[i].force1 > 0.001) && (this.nodeData[i].delta1 > 0.001)) {
                // 初期剛性 t/cm
                stiff[index][K1] = this.nodeData[i].force1 / (0.1 * this.nodeData[i].delta1) / this.G_ACC;
                if (this.plastic == BI_LINEAR) {
                    stiff[index][K2] = stiff[index][K1]; // 第2剛性 = 初期剛性
                    stiff[index][K3] = this.nodeData[i].beta * stiff[index][K1]; // 第3剛性
                    stiff[index][D2] = 0.1 * this.nodeData[i].delta1; // 第2折点の変位
                    stiff[index][D1] = 0.5 * stiff[index][D2]; // 第1折点の変位 -> 第2折点の1/2
                } else if (this.plastic == TRI_LINEAR) {
                    if ((this.nodeData[i].force2 > this.nodeData[i].force1) && (this.nodeData[i].delta2 > this.nodeData[i].delta1)) {
                        stiff[index][D1] = 0.1 * this.nodeData[i].delta1; // 第1折点の変位
                        stiff[index][D2] = 0.1 * this.nodeData[i].delta2; // 第2折点の変位
                        stiff[index][K2] = (this.nodeData[i].force2 - this.nodeData[i].force1) / (stiff[index][D2] - stiff[index][D1]) / this.G_ACC; // 第2剛性
                        stiff[index][K3] = this.nodeData[i].beta * stiff[index][K1]; // 第3剛性
                    } else {
                        this.error = "第2剛性が設定できない";
                        return 0;
                    } 
                }
            } else {
                this.error = "初期剛性が設定できない";
                return 0;
            }
        }
        for (i = 0; i < node; i++) {
            if (mass[i] < 0.1) {
                this.error = "重量の値に誤りがある";
                return 0;
            } else if (height[i] < 0.1) {
                this.error = "階高の値に誤りがある";
                return 0;
            }
        }
        // height[] 階高 -> 原点からの距離
        for (i = node - 2; i >= 0; i--) height[i] += height[i + 1];
        let wave = [];
        switch(this.wave.name) {
            case "ELCENTRO" :
                wave = waveElCentro();
                break;
            case "TAFT" :
                wave = waveTaft();
                break;
            default:
                this.error = "地震波の指定に誤りがある";
                return 0;
        }
        if (this.wave.acc > 0) {
            // 最大加速度による規準化
            let maxVal = 0;
            for (i = 1; i < wave.length; i++) {
                if (Math.abs(wave[i]) > maxVal) maxVal = Math.abs(wave[i]);
            }
            for (i = 1; i < wave.length; i++) {
                wave[i] *= this.wave.acc / maxVal;
            }
        }
        let pitch = (this.wave.pitch < 0)? wave[0] : this.wave.pitch;
        let start = (this.wave.start < 0)? 0 : this.wave.start;
        let end = (this.wave.end < 0)? pitch * wave.length : this.wave.end;
        if (start > end) start = 0;
        let acc = [];
        acc.push(0);
        let sec = 0;
        if (this.wave.pitch < 0.001) {
            // 原データの時間刻み
            for (i = 1; i < wave.length; i++) {
                if (sec > start - 0.001) acc.push(wave[i]);
                sec += pitch;
                if (sec > end + 0.001) break;
            }
        } else {
            // 時間刻みの補正
            while (sec < end + 0.001) {
                acc.push(this.getAcc(wave, pitch, sec));
                sec += pitch;
            }
        }
        // 時刻歴応答計算
        this.calcResponse(mass, height, stiff, acc, start, pitch, node);
        return 1;
    }
    calcResponse(mvect, height, stiff, acc, start, pitch, node) {
        // 時刻歴応答計算
        // mvect[]: 質量ベクトル, height[]: 基準点からの高さ, stiff[][]: 剛性
        // acc[]: 地震波の加速度, start: 開始時刻, pitch: 地震波の時間刻み, node: 質点数
        let i, j;
        let kvect = []; // 初期剛性ベクトル
        let curZone = []; // 今回の復元力 = 履歴特性の指標
        let oldZone = []; // 前回の復元力
        let bufHyst = []; // 復元力特性の折れ点の変位等を格納
        for (i = 0; i < node; i++) {
            kvect[i] = stiff[i][0];
            curZone[i] = 1;
            oldZone[i] = 0;
        }
        initArray2(bufHyst, 8);
        let kmat = []; // 剛性マトリクス
        initArray2(kmat, node);
        // 初期剛性マトリクスkmat[][]の作成
        this.setKMatrix(kvect, kmat, node);
        let cmat = []; // 減衰マトリクス
        initArray2(cmat, node);
        // 減衰マトリクスcmat[][]の作成 & 固有値計算
        this.setCMatrix(mvect, kmat, cmat, node);
        // 初期状態 xd1[]: 速度ベクトル, xd2[]: 加速度ベクトル
        let xd1 = []; // 速度
        let xd2 = []; // 加速度
        let accw = acc[1]; // 最初の地動加速度
        for (i = 0; i < node; i++) {
            xd1[i] = -accw * pitch;
        }
        let zw2 = [];
        for (i = 0; i < node; i++) {
            zw2[i] = [];
            for (j = 0; j < node; j++) {
                zw2[i][j] = -mvect[i] * cmat[i][j];
            }
        }
        let s;
        for (i = 0; i < node; i++) {
            s = 0;
            for (j = 0; j < node; j++) {
                s += zw2[i][j];
            }
            xd2[i] = (s * pitch - 1) * accw;
        }
        // 最大応答値格納用配列の初期化
        // [i][j] i: 質点番号, j = 0: 最大値, 1: 時刻
        initArray2(this.maxAcc, node, 2); // 絶対加速度
        initArray2(this.maxVel, node, 2); // 速度
        initArray2(this.maxDisp, node, 2); // 変位
        initArray2(this.maxQi, node, 2); // 層せん断力
        initArray2(this.maxSi, node, 2); // 層間変位
        initArray2(this.maxTi, node, 2); // 転倒モーメント
        initArray2(this.maxCi, node, 2); // 層せん断力係数
        initArray2(this.maxDi, node, 2); // 層間変形角
        initArray2(this.maxYi, node, 2); // 塑性率
        // 時刻歴応答
        let vectPi = []; // 換算水平力
        let vectSi = []; // 層間変位
        let ufi = []; // 不釣合力
        let fdi = []; // 不釣合力の累積
        let x00 = []; // 変位
        let xd3 = []; // 絶対加速度
        let dDi = []; // 変位の増分 ds0
        let dSi = []; // 層間変位の増分 ds1
        let curSi = []; // 今回の層間変位 hs1
        let oldSi = []; // 前回の層間変位 hs0
        initArray1(vectPi, node);
        initArray1(vectSi, node);
        initArray1(ufi, node);
        initArray1(fdi, node);
        initArray1(x00, node);
        initArray1(xd3, node);
        initArray1(dDi, node);
        initArray1(dSi, node);
        initArray1(curSi, node);
        initArray1(oldSi, node);
        let sec = start;
        let end = acc.length - 1;
        let kVerified = 0; // 1の時、剛性が変更された
        for (let nacc = 1; nacc < end; nacc++) {
            // 不釣合力の処理
            for (i = 0; i < node; i++) vectPi[i] -= ufi[i];
            // 層せん断力
            let vectQi = [];
            initArray1(vectQi, node);
            s = 0;
            for (i = 0; i < node; i++) {
                s += vectPi[i];
                vectQi[i] += s;
            }
            // 転倒モーメント
            let vectTi = []; 
            initArray1(vectTi, node);
            s = 0;
            for (i = 0; i < node; i++) {
                if (i < node -1) {
                    s += vectQi[i] * (height[i] - height[i + 1]);
                } else {
                    s += vectQi[i] * height[i];
                }
                vectTi[i] += s;
            }
            // 絶対加速度
            for (i = 0; i < node; i++) xd3[i] = xd2[i] + accw;
            // 最大値を格納
            if (nacc > 1) {
                this.setMaxVal(this.maxDisp, x00, sec, node);
                this.setMaxVal(this.maxVel, xd1, sec, node);
                this.setMaxVal(this.maxAcc, xd3, sec, node);
                this.setMaxVal(this.maxQi, vectQi, sec, node);
                this.setMaxVal(this.maxSi, vectSi, sec, node);
                this.setMaxVal(this.maxTi, vectTi, sec, node);
            }
            // t+dt の応答値
            accw = acc[nacc + 1];
            this.setKMatrix(kvect, kmat, node, 1, kVerified);
            if (this.plastic == 0) {
                // 弾性応答
                elasticNewmark(this.beta, accw, pitch, mvect, kmat, cmat, node, x00, xd1, xd2, dDi);
            } else {
                // 弾塑性応答
                plasticNewmark(this.beta, accw, pitch, mvect, kmat, cmat, node, x00, xd1, xd2, dDi, ufi, fdi);
            }
            // 層間変位 vectSi
            for (i = node - 1; i >= 0; i--) {
                vectSi[i] += dDi[i];
                if (i < node - 1) vectSi[i] -= dDi[i + 1];
            }
            // 換算水平力 vectPi
            let dp = []; // 換算水平力の増分 
            for (i = 0; i < node; i++) {
                s = 0;
                for (j = 0; j < node; j++) s += kmat[i][j] * dDi[j];
                dp[i] = s;
            }
            for (i = 0; i < node; i++) vectPi[i] += dp[i];
            // 層間変位の増分
            for (i = 0; i < node - 1; i++) dSi[i] = dDi[i] - dDi[i + 1];
            dSi[node - 1] = dDi[node - 1];
            // 今回の層間変位
            for (i = 0; i < node; i++) curSi[i] = oldSi[i] + dSi[i];
            if (this.plastic > 0) {
                // 復元力特性により次回のバネ剛性を求める
                kVerified = this.hystControl(kvect, stiff, curZone, oldZone, bufHyst, dSi, curSi, oldSi, ufi, vectQi, node);
            }
            for (i = 0; i < node; i++) oldSi[i] = curSi[i];
            sec += pitch;
        }
        // 層せん断力係数の最大値
        s = 0;
        for (i = 0; i < node; i++) {
            s += mvect[i] * this.G_ACC * 100;
            this.maxCi[i][0] = this.maxQi[i][0] / s;
            this.maxCi[i][1] = this.maxQi[i][1];
        }
        // 層間変形角の最大値
        for (i = 0; i < node; i++) {
            if (i < node - 1) {
                this.maxDi[i][0] = this.maxSi[i][0] / (height[i] - height[i + 1]);
            } else {
                this.maxDi[i][0] = this.maxSi[i][0] / height[i];
            }
            this.maxDi[i][1] = this.maxSi[i][1];
        }
        // 出力用に単位変換
        for (i = 0; i < node; i++) {
            this.maxQi[i][0] *= this.G_ACC; // せん断力 t -> kN
            this.maxTi[i][0] *= this.G_ACC * 0.01; // 転倒モーメント t.cm -> kN.m
        }
        if (this.plastic > 0) {
            // 塑性率
            for (i = 0; i < node; i++) {
                this.maxYi[i][0] = this.maxSi[i][0] / stiff[i][4];
                this.maxYi[i][1] = this.maxSi[i][1];
            }
        }
    }
    setKMatrix(kvect, kmat, node, done = 0, verified = 0) {
        // 剛性マトリクスの作成
        // kvect: 剛性ベクトル, kmat: 剛性マトリクス
        // done = 0: 初回の作成
        // verified > 0: 剛性が変更された
        if ((this.plastic == 0) || (verified == 0)) {
            if (done > 0) return;
        }
        let zw = []; 
        let zj = [];
        initArray2(zw, node); 
        initArray2(zj, node); 
        let i, j, k;
        for (i = 0; i < node; i++) {
            zw[i][i] += kvect[i];
            zj[i][i] = 1.0; 
            if (i < node - 1) {
                zw[i + 1][i] -= kvect[i];
                zj[i][i + 1] = -1.0;
            } 
        }
        initArray2(kmat, node, node);
        for (i = 0; i < node; i++) {
            for (j = 0; j < node; j++) {
                let sum = 0;
                for (k = 0; k < node; k++) {
                    sum += zw[i][k] * zj[k][j];
                }
                kmat[i][j] += sum;
            }
        }
    }
    setCMatrix(mvect, kmat, cmat, node) {
        // 減衰マトリクスの作成
        // mvect: 質量ベクトル, kmat: 剛性マトリクス, cmat: 減衰マトリクス
        // 固有値解析
        // eigen[]:i次の固有振動数, vtime[]:i次の固有周期, vmode[]:i次の固有モード
        let eigen = [];
        let vtime = [];
        let vmode = [];
        initArray1(eigen);
        initArray1(vtime);
        initArray2(vmode, node);
        eigenValue(mvect, kmat, eigen, vtime, vmode, node);
        // valMode[][]に格納
        this.setValMode(mvect, vtime, vmode, node);
        // cmat[][]の作成
        let i, j, a1, a0;
        if (this.damp.type == 0) {
            // 剛性に比例
            a1 = 2 * this.damp.damp1 / eigen[0];
            for (i = 0; i < node; i++) {
                for (j = 0; j < node; j++) {
                    cmat[i][j] = a1 * kmat[i][j];
                }
            }
        } else if (this.damp.type == 1) {
            // 質量に比例
            a1 = 2 * this.damp.damp1 * eigen[0];
            for (i = 0; i < node; i++) {
                cmat[i][i] = a1 * mvect[i];
            }
        } else {
            // レーリー減衰
            a1 = 2 * (this.damp.damp1 * eigen[0] - this.damp.damp2 * eigen[1]) / (eigen[0] * eigen[0] - eigen[1] * eigen[1]);
            a0 = 2 * this.damp.damp1 * eigen[0] - a1 * eigen[0] * eigen[0];
            for (i = 0; i < node; i++) {
                for (j = 0; j < node; j++) {
                    cmat[i][j] = a1 * kmat[i][j];
                }
                cmat[i][i] += a0 * mvect[i];
            }
        }
    }
    setValMode(mvect, vtime, vmode, node) {
        // valMode[i][j]: 次数iのモードの情報を格納
        // j = 0: 固有周期, 1: 刺激係数, 2以降: 規準化した固有モード
        let i, j;
        let nmode = (node > this.maxMode)? this.maxMode : node; // モードの最大次数
        for (i = 0; i < nmode; i++) {
            this.valMode[i] = [];
            for (j = 0; j < node + 2; j++) this.valMode[i][j] = 0;
        }
        for (i = 0; i < nmode; i++) {
            this.valMode[i][0] = vtime[i];
            for (j = 0; j < node; j++) {
                this.valMode[i][j + 2] = vmode[i][j];
            }
        }
        // 固有モードの規準化. 最大値を1とする
        let r1, r2;
        for (i = 0; i < nmode; i++) {
            r1 = 0;
            for (j = 0; j < node; j++) {
                if (Math.abs(this.valMode[i][j + 2]) > Math.abs(r1)) r1 = this.valMode[i][j + 2]; 
            }
            for (j = 0; j < node; j++) {
                this.valMode[i][j + 2] /= r1;
            }
        }
        // 刺激係数
        for (i = 0; i < nmode; i++) {
            r1 = 0;
            r2 = 0;
            for (j = 0; j < node; j++) {
                r1 += mvect[j] * this.valMode[i][j + 2];
                r2 += mvect[j] * this.valMode[i][j + 2] * this.valMode[i][j + 2];
            }
            this.valMode[i][1] = r1 / r2;
        }        
    }
    getAcc(wave, pitch, sec) {
        // 時刻 sec の加速度を直線補間により求める
        // wave[]: 加速度データの配列, pitch: wave[] の時間刻み
        let i;
        let n1 = 1;
        let n2 = 1;
        let t = 0;
        for (i = 1; i < wave.length; i++) {
            t += pitch;
            if (t < sec + 0.001) n1 = i;
            if (t > sec - 0.001) {
                n2 = i;
                break;
            }
        }
        let a1 = wave[n1];
        let a2 = wave[n2];
        if (n1 == n2) {
            return wave[n1];
        } else {
            return a1 + (a2 - a1) * (sec - n1 * pitch) / (pitch * (n2 - n1));
        }
    }
    setMaxVal(maxVal, curVal, sec, node) {
        // maxVal[i][j]: 質点iの最大値格納用配列. j = 0:値, 1:時刻
        // curVal[i]: 質点iの最大値. sec: 時刻
        for (let i = 0; i < node; i++) {
            if (Math.abs(curVal[i]) > maxVal[i][0]) {
                maxVal[i][0] = Math.abs(curVal[i]);
                maxVal[i][1] = sec;
            }
        }
    }
    hystControl(kvect, stiff, curZone, oldZone, bufHyst, dSi, curSi, oldSi, ufi, vectQi, node) {
        // 復元力特性により時刻t+dtの剛性を算出
        // kvect[]: 剛性, stiff[][]: 剛性・折点の変位, bufHyst[][]: 復元力特性の保存, 
        // curZone[]: 今回の復元力指標, oldZone[]: 前回の復元力指標, dSi[]: 層間変位の増分, 
        // curSi[]: 今回の層間変位, oldSi[]: 前回の層間変位, ufi[]: 不釣合力, vectQi[]: 層せん断力
        let i;
        let zwi = [];
        initArray1(vectQi, node);
        initArray1(ufi, node);
        initArray1(zwi, node);
        let verified = 0; // > 0: 剛性変更あり
        for (i = 0; i < node; i++) {
            let ds = dSi[i];
            let st1 = bufHyst[i][0]; // 前回の応力
            let st0 = st1 + kvect[i] * ds; // 今回の応力
            let st2 = 0;
            let flag = 0;
            while(true) {
                // 不釣合力がなくなるまで反復
                st2 = st1 + kvect[i] * ds;
                let {ufFlag, dStr} = this.hystStandard(i, dSi, curSi, oldSi, stiff, curZone, oldZone, bufHyst, kvect);
                if (ufFlag > 0) {
                    // 不釣合力がある
                    let r = Math.abs(dStr / (st2 - st1));
                    oldSi[i] = oldSi[i] + curSi[i] * r;
                    ds = ds * (1 - r);
                    st1 += dStr;
                    ++flag;
                } else {
                    break;
                }
            }
            if (flag > 0) {
                // 不釣合い力がある
                ++verified;
                zwi[i] = zwi[i] + st0 - st2;
                if (i < node - 1) zwi[i + 1] = -st0 + st2; 
            }
            bufHyst[i][0] = st2;
            vectQi[i] -= st2;
        }
        if (verified > 0) {
            for (i = 0; i < node; i++) ufi[i] += zwi[i];
        }
        return verified;
    }
    hystStandard(n, dSi, curSi, oldSi, stiff, curZone, oldZone, bufHyst, kvect) {
        // 標準型の復元力特性(Bi-Linear / Tri-Linear)による剛性と応力を求める
        // 復元力特性の各ゾーンに1から18の番号が割り当てられる
        // n: 質点のindex, dSi[]: 層間変位の増分, curSi[]: 今回の層間変位, oldSi[]: 前回の層間変位, 
        // stiff[][0]: 初期剛性, [1]:第2剛性, [2]:第1折れ点の変位, [3]:第3剛性, [4]:第2折れ点の変位,
        // curZone[]: 今回の履歴指標, oldZone[]: 前回の履歴指標, bufHyst[][]: 各折点の変位, kvect[]: バネ剛性,
        // 戻り値 ufFlag: 1の時、不釣合力がある. dStr: 応力の増分
        let ufFlag = 0; // >0の時、不釣合力がある
        let dStr = 0; // 応力の増分
        switch(curZone[n]) {
            case 1:
                if (Math.abs(curSi[n]) < stiff[n][2]) {
                    curZone[n] = 1;
                } else {
                    curZone[n] = 2;
                    ++ufFlag;
                    if (dSi[n] > 0) {
                        dStr = kvect[n] * (stiff[n][2] - oldSi[n]);
                    } else {
                        dStr = -kvect[n] * (stiff[n][2] + oldSi[n]);
                    }
                }
                break;
            case 2:
                if ((curSi[n] >= 0) && (dSi[n] < 0)) {
                    // 除荷
                    curZone[n] = 9;
                    bufHyst[n][1] = oldSi[n];
                    bufHyst[n][2] = oldSi[n] - 2 * stiff[n][2];
                    bufHyst[n][3] = oldSi[n] - 2 * stiff[n][4];
                    ++ufFlag;
                    oldZone[n] = 2;
                } else if ((curSi[n] < 0) && (dSi[n] >= 0)) {
                    // 除荷
                    curZone[n] = 10;
                    bufHyst[n][1] = oldSi[n];
                    bufHyst[n][2] = oldSi[n] + 2 * stiff[n][2];
                    bufHyst[n][3] = oldSi[n] + 2 * stiff[n][4];
                    ++ufFlag;
                    oldZone[n] = 2;
                } else if (curSi[n] >= stiff[n][4]) {
                    curZone[n] = 3;
                    ++ufFlag;
                    dStr = kvect[n] * (stiff[n][4] - oldSi[n]);
                } else if (-curSi[n] >= stiff[n][4]) {
                    curZone[n] = 6;
                    ++ufFlag;
                    dStr = -kvect[n] * (stiff[n][4] + oldSi[n]);
                }
                break;
            case 3:
                if (dSi[n] > 0) {
                    curZone[n] = 3;
                } else {
                    // 除荷
                    curZone[n] = 4;
                    bufHyst[n][1] = oldSi[n];
                    bufHyst[n][2] = oldSi[n] - 2 * stiff[n][2];
                    bufHyst[n][3] = oldSi[n] - 2 * stiff[n][4];
                    ++ufFlag;
                    oldZone[n] = 3;
                }
                break;
            case 4:
                if (curSi[n] >= bufHyst[n][1]) {
                    // 再到達
                    curZone[n] = oldZone[n];
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][1] - oldSi[n]);
                } else if (curSi[n] < bufHyst[n][2]) {
                    curZone[n] = 5;
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][2] - oldSi[n]);
                }
                break;
            case 5:
                if (curSi[n] < bufHyst[n][3]) {
                    curZone[n] = 6;
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][3] - oldSi[n]);
                } else if (dSi[n] > 0) {
                    // 除荷
                    curZone[n] = 11;
                    bufHyst[n][4] = oldSi[n];
                    bufHyst[n][5] = oldSi[n] + 2 * stiff[n][2];
                    ++ufFlag;
                    oldZone[n] = 5;
                }
                break;
            case 6:
                if (dSi[n] > 0) {
                    // 除荷
                    curZone[n] = 7;
                    bufHyst[n][1] = oldSi[n];
                    bufHyst[n][2] = oldSi[n] + 2 * stiff[n][2];
                    bufHyst[n][3] = oldSi[n] + 2 * stiff[n][4];
                    ++ufFlag;
                    oldZone[n] = 6;
                }
                break;
            case 7:
                if (curSi[n] < bufHyst[n][1]) {
                    // 再到達
                    curZone[n] = oldZone[n];
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][1] - oldSi[n]);
                } else if (curSi[n] >= bufHyst[n][2]) {
                    curZone[n] = 8;
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][2] - oldSi[n]);
                }
                break;
            case 8:
                if (curSi[n] >= bufHyst[n][3]) {
                    curZone[n] = 3;
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][3] - oldSi[n]);
                } else if (dSi[n] < 0) {
                    // 除荷
                    curZone[n] = 13;
                    bufHyst[n][4] = oldSi[n];
                    bufHyst[n][5] = oldSi[n] - 2 * stiff[n][2];
                    ++ufFlag;
                    oldZone[n] = 8;
                }
                break;
            case 9:
                if (curSi[n] >= bufHyst[n][1]) {
                    // 再到達
                    curZone[n] = oldZone[n];
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][1] - oldSi[n]);
                } else if (curSi[n] < bufHyst[n][2]) {
                    // 除荷
                    curZone[n] = 17;
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][2] - oldSi[n]);
                    oldZone[n] = 9;
                }
                break;
            case 10:
                if (curSi[n] < bufHyst[n][1]) {
                    // 再到達
                    curZone[n] = oldZone[n];
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][1] - oldSi[n]);
                } else if (curSi[n] >= bufHyst[n][2]) {
                    curZone[n] = 18;
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][2] - oldSi[n]);
                }
                break;
            case 11:
                if (curSi[n] < bufHyst[n][4]) {
                    // 再到達
                    curZone[n] = oldZone[n];
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][4] - oldSi[n]);
                } else if (curSi[n] > bufHyst[n][5]) {
                    curZone[n] = 12;
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][5] - oldSi[n]);
                }
                break;
            case 12:
                if (dSi[n] < 0) {
                    // 除荷
                    curZone[n] = 15;
                    bufHyst[n][4] = oldSi[n];
                    bufHyst[n][2] = oldSi[n] - 2 * stiff[n][2];
                    ++ufFlag;
                    oldZone[n] = 12;
                } else if (curSi[n] > bufHyst[n][1]) {
                    curZone[n] = 3;
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][1] - oldSi[n]);
                }
                break;
            case 13:
                if (curSi[n] >= bufHyst[n][4]) {
                    // 再到達
                    curZone[n] = oldZone[n];
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][4] - oldSi[n]);
                } else if (curSi[n] < bufHyst[n][5]) {
                    curZone[n] = 14;
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][5] - oldSi[n]);
                }
                break;
            case 14:
                if (dSi[n] > 0) {
                    // 除荷
                    curZone[n] = 16;
                    bufHyst[n][4] = oldSi[n];
                    bufHyst[n][2] = oldSi[n] + 2 * stiff[n][2];
                    ++ufFlag;
                    oldZone[n] = 14;
                } else if (curSi[n] < bufHyst[n][1]) {
                    curZone[n] = 6;
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][1] - oldSi[n]);
                }
                break;
            case 15:
                if (curSi[n] >= bufHyst[n][1]) {
                    // 再到達
                    curZone[n] = oldZone[n];
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][1] - oldSi[n]);
                } else if (curSi[n] >= bufHyst[n][4]) {
                    // 再到達
                    curZone[n] = oldZone[n];
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][4] - oldSi[n]);
                } else if (curSi[n] < bufHyst[n][2]) {
                    curZone[n] = 5;
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][2] - oldSi[n]);
                }
                break;
            case 16:
                if (curSi[n] < bufHyst[n][1]) {
                    curZone[n] = 6;
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][1] - oldSi[n]);
                } else if (curSi[n] < bufHyst[n][4]) {
                    // 再到達
                    curZone[n] = oldZone[n];
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][4] - oldSi[n]);
                } else if (curSi[n] >= bufHyst[n][2]) {
                    curZone[n] = 8;
                    ++ufFlag;
                    dStr = kvect[n] * (bufHyst[n][2] - oldSi[n]);
                }
                break;
            case 17:
                if (-curSi[n] >= stiff[n][4]) {
                    curZone[n] = 6;
                    ++ufFlag;
                    dStr = -kvect[n] * (stiff[n][4] + oldSi[n]);
                } else if (dSi[n] > 0) {
                    // 除荷
                    curZone[n] = 10;
                    bufHyst[n][1] = oldSi[n];
                    bufHyst[n][2] = oldSi[n] + 2 * stiff[n][2];
                    bufHyst[n][3] = oldSi[n] + 2 * stiff[n][4];
                    ++ufFlag;
                    oldZone[n] = 17;
                }
                break;
            case 18:
                if (curSi[n] >= stiff[n][4]) {
                    curZone[n] = 3;
                    ++ufFlag;
                    dStr = kvect[n] * (stiff[n][4] - oldSi[n]);
                } else if (dSi[n] < 0) {
                    // 除荷
                    curZone[n] = 9;
                    bufHyst[n][1] = oldSi[n];
                    bufHyst[n][2] = oldSi[n] - 2 * stiff[n][2];
                    bufHyst[n][3] = oldSi[n] - 2 * stiff[n][4];
                    ++ufFlag;
                    oldZone[n] = 18;
                }
                break;
        }
        switch(curZone[n]) {
            case 1:
            case 4:
            case 7:
            case 9:
            case 10:
            case 11:
            case 13:
            case 15:
            case 16:
                kvect[n] = stiff[n][0];
                break;
            case 2:
            case 5:
            case 8:
            case 12:
            case 14:
            case 17:
            case 18:
                kvect[n] = stiff[n][1];
                break;
            default:
                kvect[n] = stiff[n][3];
        }
        return {ufFlag, dStr};
    }
    getMaxValue(id = 0, contents = 0) {
        // 戻り値: 最大値をあらわす文字列 = 階名 + 最大値 + (発生時刻sec)
        // id: 階のIDまたは名称
        // contents = 1: 加速度(gal), 2: 速度(cm/sec), 3: 絶対変位(cm), 4: 層せん断力(kN)
        //            5: 層せん断力係数, 6: 層間変位(cm), 7: 層間変形角(分数),
        //            8: 転倒モーメント(kN.m), 9: 塑性率
        let index = -1;
        let result = ""
        for (let i = 0; i < this.maxNode; i++) {
            if ((this.nodeData[i].id == id) || (this.nodeData[i].name == id)) {
                index = this.nodeData[i].id;
            } 
        }
        if (index <= 0) return result;
        index -= 1;
        let val, sec;
        switch(contents) {
            case 1:
                val = this.maxAcc[index][0].toFixed(2);
                sec = this.maxAcc[index][1];
                break;
            case 2:
                val = this.maxVel[index][0].toFixed(2);
                sec = this.maxVel[index][1];
                break;
            case 3:
                val = this.maxDisp[index][0].toFixed(3);
                sec = this.maxDisp[index][1];
                break;
            case 4:
                val = this.maxQi[index][0].toFixed(1);
                sec = this.maxQi[index][1];
                break;
            case 5:
                val = this.maxCi[index][0].toFixed(3);
                sec = this.maxCi[index][1];
                break;
            case 6:
                val = this.maxSi[index][0].toFixed(3);
                sec = this.maxSi[index][1];
                break;
            case 7:
                val = "1/" + (1 / this.maxDi[index][0]).toFixed();
                sec = this.maxDi[index][1];
                break;
            case 8:
                val = this.maxTi[index][0].toFixed();
                sec = this.maxTi[index][1];
                break;
            case 9:
                val = this.maxYi[index][0].toFixed(3);
                sec = this.maxYi[index][1];
                break;
            default:
                return result;
        }
        result = this.nodeData[index].name + " " + val + " (" + sec.toFixed(3) + ")";
        return result;

    }
}
function eigenValue(mvect, kmat, eigen, vtime, vmode, node) {
    // 固有値の計算
    // mvect: 質量ベクトル, kmat: 剛性マトリクス, node: 質点数
    // eigen[]:i次の固有円振動数(rad), vtime[]:i次の固有周期, 
    // vmode[]:i次の固有モード
    let i, j;
    let vect = [];
    eigen[node - 1] = eigenMax(mvect, kmat, node, vect); // 最大固有値
    for (i = 0; i < node; i++) vmode[node - 1][i] = vect[i];
    eigen[0] = eigenMin(mvect, kmat, node, vect, 0); // 最小固有値
    for (i = 0; i < node; i++) vmode[0][i] = vect[i];
    let n1, n2, flag, mode;
    let ramda, shift;
    while (true) {
        n1 = -1;
        n2 = -1;
        flag = 0;
        for (i = 0; i < node; i++) {
            if (eigen[i] > 0.0001) {
                n1 = i;
            } else {
                for (j = i + 1; j < node; j++) {
                    if (eigen[j] > 0.0001) {
                        n2 = j;
                        flag += 1;
                        break;
                    }
                }
                if (flag > 0) break;
            }
        }
        if ((n1 >= (node - 1)) || (flag == 0)) break;
        shift = (eigen[n1] + eigen[n2]) / 2;
        ramda = eigenMin(mvect, kmat, node, vect, shift);
        mode = 0; // 固有モードの次数 = 基準軸と交差する回数
        for (i = 0; i < (node - 1); i++) {
            n1 = (vect[i] > 0)? 0 : 1;
            n2 = (vect[i + 1] > 0)? 0 : 1;
            if (n1 != n2) mode += 1;
        }
        for (i = 0; i < node; i++) vmode[mode][i] = vect[i];
        eigen[mode] = ramda;
    }
    for (i = 0; i < node; i++) {
        eigen[i] = Math.sqrt(eigen[i]); // 固有値 -> 固有振動数
        vtime[i] = 2 * Math.PI / eigen[i]; // 固有振動数 -> 固有周期
    }
}
function eigenMax(mvect, kmat, node, vect) {
    // 最大固有値の計算
    // mvect: 質量ベクトル, kmat: 剛性マトリクス, node: 質点数, vect: 固有ベクトル
    // 戻り値: 最大固有値
    let ramda = 0; // 最大固有値
    let x = [];
    let y = [];
    let mv = [];
    let i, j;
    for (i = 0; i < node; i++) {
        x[i] = 1;
    }
    for (i = 0; i < node; i++) {
        y[i] = 0;
        for (j = 0; j < node; j++) y[i] += kmat[i][j] * x[j];
        mv[i] = 1.0 / mvect[i];
    }
    let eps = 0.001;
    let num1, num2, r, gap;
    let yy = [];

    while (true) {
        for (i = 0; i < node; i++) x[i] = y[i] * mv[i];
        for (i = 0; i < node; i++) {
            yy[i] = 0;
            for (j = 0; j < node; j++) yy[i] += kmat[i][j] * x[j];
        }
        num1 = 0; // 分母
        num2 = 0; // 分子
        for (i = 0; i < node; i++) {
            num1 += x[i] * y[i];
            num2 += x[i] * yy[i];
        }
        r = num2 / num1;
        gap = Math.abs(r - ramda);
        ramda = r;
        num1 = 1.0 / Math.sqrt(num1);
        for (i = 0; i < node; i++) y[i] = yy[i] * num1;
        if (gap < eps) break;
    }
    for (i = 0; i < node; i++) vect[i] = x[i] * num1;
    return ramda;
}
function eigenMin(mvect, kmat, node, vect, shift) {
    // 最小固有値の計算
    // mvect: 質量ベクトル, kmat: 剛性マトリクス, node: 質点数, vect: 固有ベクトル
    // shift: 最小固有値の場合は0
    // 戻り値: 最小固有値
    let ramda = 0; // 最小固有値
    let x = [];
    let y = [];
    let mat = [];
    initArray2(mat, node);
    let i, j;
    for (i = 0; i < node; i++) {
        x[i] = 1;
        y[i] = mvect[i];
    }
    for (i = 0; i < node; i++) {
        for (j = 0; j < node; j++) {
            mat[i][j] = kmat[i][j];
        }
        mat[i][i] -= shift * mvect[i];
    }
    let inv = [];
    inv = invMatrix(mat, node);
    let eps = 0.001;
    let num1, num2, r, gap;
    let yy = [];

    while (true) {
        for (i = 0; i < node; i++) {
            x[i] = 0;
            for (j = 0; j < node; j++) x[i] += inv[i][j] * y[j];
        }
        for (i = 0; i < node; i++) {
            yy[i] = mvect[i] * x[i];
        }
        num1 = 0; // 分母
        num2 = 0; // 分子
        for (i = 0; i < node; i++) {
            num1 += x[i] * yy[i];
            num2 += x[i] * y[i];
        }
        r = num2 / num1;
        gap = Math.abs(r - ramda);
        ramda = r;
        num1 = 1 / Math.sqrt(num1);
        for (i = 0; i < node; i++) y[i] = yy[i] * num1;
        if (gap < eps) break;
    }
    for (i = 0; i < node; i++) vect[i] = x[i] * num1;
    return (ramda + shift);
}
function invMatrix(a, node) {
    // 戻り値: a[node][node]の逆マトリクス
    let inv = [];
    let i, j, k, n;
    for (i = 0; i < node; i++) {
        inv[i] = [];
        for (j = 0; j < node; j++) {
            inv[i][j] = a[i][j];
        }
    }
    let c;
    for (j = 1; j < node; j++) {
        for (i = 1; i < j; i++) {
            for (k = 0; k < i; k++) inv[i][j] -= inv[k][i] * inv[k][j];
        }
        for (i = 0; i < j; i++) {
            c = inv[i][j] / inv[i][i];
            inv[j][j] -= c * inv[i][j];
            inv[i][j] = c;
        }
    }
    let d = [];
    for (n = 0; n < node; n++) {
        let r = [];
        initArray1(r, node);
        r[n] = 1.0;
        for (i = n + 1; i < node; i++) {
            for (k = 0; k < i; k++) r[i] -= inv[k][i] * r[k];
        }
        for (i = 0; i < node; i++) r[i] /= inv[i][i];
        for (i = node - 1; i > n; i--) {
            for (k = 0; k < i; k++) {
                r[k] -= inv[k][i] * r[i];
            }
        }
        for (i = n + 1; i < node; i++) inv[i][n] = r[i];
        d[n] = r[n];
    }
    for (i = 0; i < node; i++) {
        inv[i][i] = d[i];
        for (j = i + 1; j < node; j++) inv[i][j] = inv[j][i];
    }
    
    return inv;
}
function elasticNewmark(beta, acc, dt, mvect, kmat, cmat, node, x00, xd1, xd2, d00) {
    // Newmark法による弾性応答計算
    // beta: β, acc: 地動加速度, dt: 時間刻み, mvect[]: 質量ベクトル, cmat[][]: 減衰マトリクス, 
    // kmat[][]: 剛性マトリクス, node: 質点数
    // 時刻t+tdにおける応答値
    // x00[]: 変位, xd1[]: 速度, xd2[]: 加速度, d00[]: 変位の増分
    let i, j, k;
    let bk = [];
    let s;
    for (i = 0; i < node; i++) {
        bk[i] = [];
        for (j = 0; j < node; j++) {
            s = (i == j)? mvect[i] : 0;
            bk[i][j] = kmat[i][j] + s / (beta * dt * dt) + cmat[i][j] / (2 * beta * dt);
        }
    }
    let bp = [];
    for (i = 0; i < node; i++) {
        bp[i] = -mvect[i] * acc;
    }
    let zw = [];
    for (i = 0; i < node; i++) {
        zw[i] = [];
        for (j = 0; j < node; j++) {
            s = (i == j)? mvect[i] : 0;
            zw[i][j] = s / (beta * dt * dt) + cmat[i][j] / (2 * beta * dt);
        }
    }
    for (i = 0; i < node; i++) {
        s = 0;
        for (k = 0; k < node; k++) s += zw[i][k] * x00[k];
        bp[i] += s; 
    }
    for (i = 0; i < node; i++) {
        for (j = 0; j < node; j++) {
            s = (i == j)? mvect[i] : 0;
            zw[i][j] = s / (beta * dt) + cmat[i][j] / (1 / (2 * beta) - 1)
        }
    }
    for (i = 0; i < node; i++) {
        s = 0;
        for (k = 0; k < node; k++) s += zw[i][k] * xd1[k];
        bp[i] += s; 
    }
    for (i = 0; i < node; i++) {
        for (j = 0; j < node; j++) {
            s = (i == j)? mvect[i] : 0;
            zw[i][j] = s * (1 / (2 * beta) - 1) + cmat[i][j] * (dt / 2) * (1 / (2 * beta) - 2);
        }
    }
    for (i = 0; i < node; i++) {
        s = 0;
        for (k = 0; k < node; k++) s += zw[i][k] * xd2[k];
        bp[i] += s; 
    }
   
    bk = invMatrix(bk, node);
    for (i = 0; i < node; i++) {
        let d0 = x00[i];
        let d1 = xd1[i];
        let d2 = xd2[i];
        s = 0;
        for (k = 0; k < node; k++) {
            s += bk[i][k] * bp[k];
        }
        x00[i] = s;
        xd1[i] = (1 / (2* beta * dt)) * (x00[i] - d0) + (1 - 1 / (2 * beta)) * d1 + dt * (1 - 1 / (4 * beta)) * d2;
        xd2[i] = (1 / (beta * dt * dt)) * (x00[i] - d0) - (1 / (beta * dt)) * d1 - (1 / (2 * beta) - 1) * d2;
        d00[i] = s - d0;
    }
}
function plasticNewmark(beta, acc, dt, mvect, kmat, cmat, node, x00, xd1, xd2, d00, ufi, fdi) {
    // Newmark法による弾塑性応答計算
    // beta: β, acc: 地動加速度, dt: 時間刻み, mvect[]: 質量ベクトル, cmat[][]: 減衰マトリクス, 
    // kmat[][]: 剛性マトリクス, node: 質点数
    // 時刻t+tdにおける応答値
    // x00[]: 変位, xd1[]: 速度, xd2[]: 加速度, d00[]: 変位の増分, ufi[]: 不釣合力, fdi[]: 累積値
    let i, j, k;
    let bk = [];
    let s;
    for (i = 0; i < node; i++) {
        bk[i] = [];
        for (j = 0; j < node; j++) {
            s = (i == j)? mvect[i] : 0;
            bk[i][j] = s + cmat[i][j] * dt / 2 + kmat[i][j] * beta * dt * dt;
        }
    }
    let bp = [];
    for (i = 0; i < node; i++) {
        bp[i] = -mvect[i] * acc + ufi[i] - fdi[i];
    }
    let zw = [];
    for (i = 0; i < node; i++) {
        zw[i] = [];
        for (j = 0; j < node; j++) {
            zw[i][j] = cmat[i][j] * dt / 2 + kmat[i][j] * (0.5 - beta) * dt * dt;
        }
    }
    for (i = 0; i < node; i++) {
        s = 0;
        for (k = 0; k < node; k++) s += zw[i][k] * xd2[k];
        bp[i] -= s; 
    }
    for (i = 0; i < node; i++) {
        for (j = 0; j < node; j++) {
            zw[i][j] = cmat[i][j] + kmat[i][j] * dt;
        }
    }
    for (i = 0; i < node; i++) {
        s = 0;
        for (k = 0; k < node; k++) s += zw[i][k] * xd1[k];
        bp[i] -= s; 
    }
    bk = invMatrix(bk, node);
    for (i = 0; i < node; i++) {
        let d1 = xd1[i];
        let d2 = xd2[i];
        let d3 = 0;
        for (k = 0; k < node; k++) {
            d3 += bk[i][k] * bp[k];
        }
        let d0 = dt * d1 + (0.5 - beta) * dt * dt * d2 + beta * dt * dt * d3;
        xd1[i] += 0.5 * dt * (d2 + d3);
        xd2[i] = d3;
        x00[i] += d0;
        d00[i] = d0;
    }
    for (i = 0; i < node; i++) {
        for (j = 0; j < node; j++) {
            fdi[i] += kmat[i][j] * d00[j];
        }
        fdi[i] -= ufi[i];
    }
}
function initArray1(array1, n) {
    // 要素数 n の1次元配列 array1 の初期化
    for (let i = 0; i < n; i++) {
        array1[i] = 0;
    }
}
function initArray2(array2, n1, n2 = 0) {
    // 要素数 n1*n2 の2次元配列 array2 の初期化
    let elem = (n2 == 0)? n1 : n2;
    for (let i = 0; i < n1; i++) {
        array2[i] = [];
        for (let j = 0; j < elem; j++) array2[i][j] = 0;
    }
}
function waveElCentro() {
    // EL-CENTRO NS の加速度データ
    // 建築性能基準推進協会のウェブサイトで配布 1968年版「ビルディング・レター」
    // 時間刻み 0.02秒, 最大加速度 341.7gal, 最大速度 33.5kine 
    // 最初のデータは時間刻み(sec), 以降は加速度
    // データ数 385 (7.7秒)
    let wave = [0.02, // 時間刻み
     -1.4,     -10.8,     -10.1,      -8.8,      -9.5,     -12.0,     -14.2,
    -12.8,     -11.0,      -8.5,      -8.5,     -13.1,     -17.6,     -19.4,
    -16.2,     -14.4,     -10.8,      -8.2,      -4.2,      -6.6,     -13.1,
    -19.0,     -19.6,      -6.6,       3.0,      14.1,      -4.9,     -12.8,
    -14.4,     -20.3,     -26.0,     -32.5,     -30.6,     -17.2,     -19.7,
    -16.3,     -16.4,      -6.7,       2.5,      15.0,      23.6,      25.2,
     33.6,      46.3,      49.2,      41.9,      35.9,      27.1,      23.5,
     33.9,      41.2,      53.0,      63.9,      73.2,      65.2,      59.9,
     40.0,      40.0,       6.3,     -51.5,     -78.7,     -60.3,     -48.4,
    -25.0,      -5.9,      13.4,      30.8,      49.9,      71.0,      99.5,
    121.9,     152.9,     144.9,     115.5,      93.5,      89.2,      92.6,
     83.9,      90.1,      99.3,     120.9,      32.8,    -147.5,    -206.6,
   -198.9,    -203.4,    -181.6,    -172.5,    -175.2,    -175.3,    -180.5,
   -163.0,    -134.7,    -108.7,     -78.2,     -42.9,      -1.7,      36.0,
     78.5,     116.4,     159.8,     196.0,     241.2,     272.9,     303.6,
    320.0,     341.7,     282.1,     232.4,    -119.8,    -237.3,    -164.0,
   -186.5,    -109.5,     -75.3,     -17.3,      11.3,      53.3,      89.5,
    118.6,     175.7,      57.6,    -263.1,    -154.7,    -172.9,    -101.2,
    -57.9,      23.7,     -67.0,    -198.0,    -164.1,    -168.5,    -148.1,
   -123.1,    -100.1,     -75.1,     -52.3,     -27.1,      -4.4,      18.8,
     -9.5,     -43.3,     -83.8,     -95.1,     -71.6,     -59.9,     -33.4,
    -10.8,      18.5,      42.0,      67.3,      -9.7,     -37.2,      -4.0,
      1.1,      34.4,      56.5,      88.3,     113.0,     136.3,      21.9,
     24.1,      68.3,      68.9,     131.8,     135.3,     204.0,     -93.1,
   -130.8,     -69.2,     -54.6,       7.2,      67.5,    -106.7,    -148.8,
   -107.1,    -116.2,     -76.2,     -55.9,     -21.5,     -12.6,     -67.4,
    -32.4,     -33.7,     -10.9,       1.7,      29.9,      48.8,      60.8,
     22.2,      -3.2,     -24.5,       7.7,      21.1,      56.8,      82.6,
    120.6,     147.8,     173.7,      42.1,       2.9,      25.9,      29.3,
     -5.5,     -14.7,      14.3,      20.6,      49.9,      64.5,      95.7,
    112.8,     144.7,     162.9,     194.5,     185.6,     198.4,     176.9,
    125.0,    -120.7,     -54.2,     -38.4,     -31.1,    -111.8,    -166.1,
   -246.4,    -202.5,    -183.5,    -131.7,     -96.0,     -32.5,      15.4,
     81.6,     131.9,     181.8,      -5.8,     -16.9,      28.5,      44.7,
     98.3,     142.4,     185.3,     245.6,     168.5,    -138.0,     -99.9,
   -108.9,     -90.7,     -46.9,    -125.0,    -211.1,    -161.7,    -169.2,
   -130.6,    -111.1,     -77.3,     -51.0,     -54.4,    -120.0,    -120.9,
   -115.8,    -114.5,     -71.7,     -54.6,       6.4,     -80.4,    -163.4,
    -85.9,     -96.1,     -39.6,     -14.7,      31.9,      64.8,      87.6,
     47.2,      19.8,      -2.7,      29.2,      44.5,      78.5,     103.3,
    135.2,     160.6,     186.1,     128.1,      64.0,      20.4,      31.4,
     37.3,      49.6,      23.5,      -8.4,     -16.8,     -11.3,     -22.9,
    -24.8,     -15.7,      -6.9,      14.7,      37.9,      57.9,      25.5,
     -4.1,     -42.8,     -13.3,       9.5,      23.0,     -12.9,      -5.0,
      8.0,      21.0,      38.0,      51.0,      15.7,      -3.2,     -11.1,
      0.5,       7.6,       3.5,      -9.5,      -3.6,      -1.6,       3.8,
      8.5,      -5.6,     -30.4,     -42.1,     -24.4,     -23.6,     -17.7,
    -12.9,      -1.8,      20.3,     -10.8,      -9.1,      -3.4,     -10.6,
    -11.1,      -9.9,      -0.2,       7.3,      23.5,      35.5,      70.5,
     77.9,      18.4,     -26.3,     -12.4,      -4.2,      15.9,       4.8,
    -21.9,     -46.7,     -42.8,     -21.6,      -4.3,      15.9,      32.0,
     41.9,      12.3,     -16.0,     -20.4,      -8.2,     -20.6,     -13.7,
     -5.5,       5.3,      13.4,      26.6,      23.2,       7.9,      -0.8,
     20.0,      43.5,      49.2,      19.1,       9.2,      -2.2,      -2.1,
      5.2,       9.3,      25.5,      36.8,      52.5,      54.1,      42.5];
    
    return wave;
}
function waveTaft() {
    // TAFT EW の加速度データ
    // 建築性能基準推進協会のウェブサイトで配布 1968年版「ビルディング・レター」
    // 時間刻み 0.02秒, 最大加速度 175.9gal, 最大速度 17.7kine 
    // 最初のデータは時間刻み(sec), 以降は加速度
    // データ数 385 (7.7秒)
    let wave = [0.02, // 時間刻み
      -6.2,      -1.9,       4.0,       9.9,       5.2,      -0.3,      -4.2,
      -2.8,       1.2,       5.3,       3.9,      -3.0,      -8.5,      -8.5,
      -6.0,      -0.8,      -0.3,      -0.2,       4.0,      10.5,      11.4,
       6.5,       4.4,       2.3,       2.1,       1.9,      -3.9,      -7.4,
      -1.1,       5.8,       2.8,     -10.2,     -11.3,      -4.7,       4.9,
       1.0,      -4.0,      -2.5,       5.8,       7.2,      -6.5,     -10.7,
       3.2,      16.9,       9.9,      -4.8,      -6.5,       3.6,       4.5,
      -2.8,       3.5,      14.3,      22.8,      17.8,      18.3,      12.3,
      -1.3,      -8.5,      -6.7,      -4.7,       4.0,       5.5,      -1.4,
      -4.0,      -0.1,      -2.2,      -8.5,     -11.1,     -15.0,     -18.4,
     -15.7,      -7.9,      -0.3,       8.1,       4.2,      -7.7,     -18.0,
     -11.4,       0.9,      10.5,       8.5,       7.4,       8.8,       9.6,
       1.6,      -5.2,      -0.8,       6.7,      16.8,      19.5,      13.5,
       5.0,       3.1,      11.7,      23.4,      23.7,       6.2,      -6.8,
     -11.2,      -8.5,      -3.8,       2.2,       2.3,     -15.8,     -26.0,
      -9.0,      16.0,      21.9,      16.7,      21.0,      24.3,      18.4,
       4.2,     -10.4,     -23.4,      -8.8,      13.9,      36.2,      30.3,
      15.7,      11.3,       7.2,      -7.9,     -13.2,     -12.5,      -7.6,
       6.7,      22.2,      22.5,      19.5,      25.4,      23.2,      11.8,
      -1.0,      -8.5,     -11.2,     -14.3,     -20.4,     -19.6,     -16.6,
     -18.2,     -17.5,      -8.8,      -1.5,      -4.9,      -5.2,      -4.0,
      -9.1,      -7.8,      -2.1,       3.4,      -5.0,     -21.8,     -35.6,
     -32.0,     -24.0,     -32.2,     -48.3,     -56.7,     -60.9,     -70.8,
     -65.1,     -53.5,     -51.6,     -48.8,     -49.8,     -51.8,     -56.1,
     -48.1,     -34.7,     -25.5,     -16.9,     -14.5,     -25.6,     -39.5,
     -45.0,     -43.2,     -23.6,       3.3,      32.1,      54.6,      74.4,
      92.4,     118.2,     146.4,     175.9,     160.7,     117.2,      68.1,
      18.4,     -34.4,     -64.0,     -61.1,     -55.3,     -45.1,     -46.8,
     -55.3,     -64.7,     -56.3,     -37.0,     -16.2,       4.6,       4.6,
      -8.6,     -22.3,     -39.5,     -45.8,     -21.0,      13.4,      49.8,
      83.4,      67.5,      36.6,      -1.5,      -1.5,      30.0,      65.2,
      74.9,      69.3,      61.8,      33.5,       3.2,     -31.7,     -44.8,
     -44.1,     -16.3,      35.0,      92.1,     121.1,      82.2,      33.5,
     -21.6,     -37.0,     -53.0,     -55.4,     -36.6,     -14.3,      10.9,
      34.2,      60.9,      68.2,      45.6,      21.0,       0.5,     -31.2,
     -84.7,    -111.2,     -86.8,     -50.0,      -9.6,      30.8,      39.2,
      27.6,      11.0,       5.0,       0.6,     -18.4,     -39.7,     -51.6,
     -41.8,     -28.5,     -17.9,     -27.1,     -40.3,     -56.4,     -66.9,
     -45.5,     -15.0,      18.9,      52.1,      86.7,     108.8,     112.0,
     102.7,      87.9,      62.0,      27.2,      -8.3,     -43.1,     -49.7,
     -41.1,     -36.7,     -47.2,     -61.5,     -77.0,     -91.2,     -72.6,
     -34.9,       7.0,      42.3,      29.9,       1.9,     -14.9,       7.4,
      36.1,      71.7,      88.8,      62.8,      29.3,     -11.7,     -21.3,
       4.5,      23.9,      32.2,      41.1,      36.1,      22.9,       7.9,
      -6.6,     -15.1,     -14.7,       0.9,      28.8,      32.3,      16.2,
      -6.8,      -2.1,      33.6,      74.7,      93.9,      71.8,      62.0,
      31.8,      -7.8,     -51.5,     -94.8,    -136.9,    -146.4,    -143.3,
    -138.4,    -133.9,    -134.1,    -134.3,    -109.0,     -76.1,     -40.9,
      -5.2,      29.7,      66.9,      83.1,      57.6,      28.8,      -9.3,
     -22.1,      -8.6,       8.7,      27.9,      48.2,      55.1,      36.2,
      14.1,     -11.5,     -36.0,     -58.8,     -51.0,     -32.6,     -11.4,
      10.5,      31.8,      54.0,      74.7,      98.6,     101.0,      69.4,
      34.4,      -6.7,     -32.0,     -12.6,       0.9,     -15.5,     -38.8,
     -58.1,     -37.9,     -16.5,     -22.1,     -39.1,     -38.1,     -13.6,
      11.7,      41.4,      67.8,      98.1,     110.3,     106.6,      99.4];
    
    return wave;
}
